#include <stdio.h>
#include <getopt.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include "lj3100.h"

#define BUFFERSIZE 256

unsigned char status_request[] = {0x5a, 0x51, 0x02, 0x00,
				  0x00, 0x00, 0x00, 0x00};

#define SWAP_INT16(obj, member) (obj->member =\
					((obj->member & 0xff00) >> 8) |\
					((obj->member & 0x00ff) << 8))

#define SWAP_INT32(obj, member) (obj->member =\
					((obj->member & 0xff000000) >> 24) |\
					((obj->member & 0x00ff0000) >>  8) |\
					((obj->member & 0x0000ff00) <<  8) |\
					((obj->member & 0x000000ff) << 24))

void
lj3100_swap_status(struct lj3100_status *status) {
	int i;
	char *byte = (char*) status;
	char tmp;

	for (i = 0; i < sizeof(struct lj3100_status); i += 2) {
		tmp = byte[i];
		byte[i] = byte[i + 1];
		byte[i + 1] = tmp;
	}

	SWAP_INT32(status, endianess1);
	SWAP_INT32(status, endianess2);

	SWAP_INT32(status, unknown08);
	SWAP_INT32(status, unknown0c);

	SWAP_INT32(status, scanned);
	SWAP_INT32(status, scanned_copy);
	SWAP_INT32(status, printed);
	SWAP_INT32(status, sent_ec);
	SWAP_INT32(status, sent_noec);
	SWAP_INT32(status, received_ec);
	SWAP_INT32(status, received_noec);
	SWAP_INT32(status, scan_errors);
	SWAP_INT32(status, print_errors);
	SWAP_INT32(status, send_ec_errors);
	SWAP_INT32(status, send_noec_errors);
	SWAP_INT32(status, receive_ec_errors);
	SWAP_INT32(status, receive_noec_errors);

	SWAP_INT32(status, time);

	SWAP_INT32(status, endianess3);
	SWAP_INT32(status, endianess4);

	SWAP_INT32(status, unknown50);
	SWAP_INT32(status, unknown54);
	SWAP_INT32(status, unknown58);
	SWAP_INT32(status, unknown5c);
	SWAP_INT32(status, unknown60);
	SWAP_INT32(status, unknown64);
	SWAP_INT32(status, unknown68);
	SWAP_INT32(status, unknown6c);
	SWAP_INT32(status, unknown70);
	SWAP_INT32(status, unknown74);
	SWAP_INT32(status, unknown78);
	SWAP_INT32(status, unknown7c);
	SWAP_INT32(status, unknown80);
	SWAP_INT32(status, unknown84);
	SWAP_INT32(status, unknown88);
	SWAP_INT32(status, unknown8c);
	SWAP_INT32(status, unknown90);
	SWAP_INT32(status, unknown94);

	SWAP_INT16(status, unknown98);
}

main(int argc, char *argv[])
{
	struct sockaddr_un sa = {AF_UNIX, "/tmp/lj3100_5"};
	struct lj3100_status status;
	int i, ret, arg, fd, size;

	fd = socket(PF_UNIX, SOCK_STREAM, 0);
	ret = connect(fd, &sa, sizeof(sa));
	if (ret == -1)
		perror("connect");

	write(fd, status_request, sizeof(status_request));

	read(fd, &status, 8);
	size = read(fd, &status, sizeof(status));
	if (size == -1)
		perror("read");

	if (size != sizeof(struct lj3100_status)) {
		printf("wrong size: %x\n", size);
		return -1;;
	}

	lj3100_swap_status(&status);


#define PRINT(memb)    printf("%20s %u\n", #memb ":", status.memb)
#define PRINTX16(memb) printf("%20s 0x%4.4x\n", #memb ":", status.memb)
#define PRINTX32(memb) printf("%20s 0x%8.8x\n", #memb ":", status.memb)
#define PRINTS(memb)   printf("%20s %s\n", #memb ":", status.memb)

	PRINTX32(endianess1);
	PRINTX32(endianess2);

	PRINTX32(unknown08);
	PRINTX32(unknown0c);

	PRINT(scanned);
	PRINT(scanned_copy);
	PRINT(printed);
	PRINT(sent_ec);
	PRINT(sent_noec);
	PRINT(received_ec);
	PRINT(received_noec);
	PRINT(scan_errors);
	PRINT(print_errors);
	PRINT(send_ec_errors);
	PRINT(send_noec_errors);
	PRINT(receive_ec_errors);
	PRINT(receive_noec_errors);

	PRINT(time);

	PRINTX32(endianess3);
	PRINTX32(endianess4);

	PRINTX32(unknown50);
	PRINTX32(unknown54);
	PRINTX32(unknown58);
	PRINTX32(unknown5c);
	PRINTX32(unknown60);
	PRINTX32(unknown64);
	PRINTX32(unknown68);
	PRINTX32(unknown6c);
	PRINTX32(unknown70);
	PRINTX32(unknown74);
	PRINTX32(unknown78);
	PRINTX32(unknown7c);
	PRINTX32(unknown80);
	PRINTX32(unknown84);
	PRINTX32(unknown88);
	PRINTX32(unknown8c);
	PRINTX32(unknown90);
	PRINTX32(unknown94);

	PRINTX16(unknown98);

	PRINTS(message[0]);
	PRINTS(message[1]);
	printf("\n");
	close(fd);
}
